/*==================================================================================*/
/* @(#)Rev.1.12 - gidadcctl.c                                                          */
/* GID-ADC Control Library for PC-UNIX. (Only 'root' can perform this.)             */
/* Copyright 1996-2003 (C) by Mathematical Assist Design Lab., all rights reserved. */
/* Copyright 1999-2003 (C) by Masahiko Watanabe, all rights reserved.               */
/* Copyright 2000 (C) by Yukio Mitsuiwa, all rights reserved.                       */
/*==================================================================================*/

/* Initial Dummy Loop Times. */
#define  INIT_DMY_LOOP   50000L  

/* Machine Switch should be specified Only PCAT by PC-UNIX(Linux/FreeBSD). */
/* (Plamo-Linux will do on PC-98x1 Series.) */

#ifdef PC98
  #undef PCAT
#endif
#ifdef PCAT
  #undef PC98
#endif
#ifdef FreeBSD
  #undef LINUX
#endif
#ifdef LINUX
  #undef FreeBSD
#endif
#ifdef LIBC5
  #undef GLIBC
#endif
#ifdef GLIBC
  #undef LIBC5
#endif

/*==================================================*/
/* Compiler Switch (Only GCC specified in PC-UNIX) #*/
#define  ASM_IO_H  0  /* Include or NOT */
#define  ASM_IOCTL_H  0  /* Include or NOT */
#define  GCC   /* For GNU C */
/* #define BC  */   /* For Borland C */
/* #define MSC */   /* For Microsoft C */
/*==================================================*/

#include <stdio.h>

#ifdef GCC
  #ifdef LINUX
    #ifdef LIBC5
      #include <unistd.h>
    #endif
    #ifdef GLIBC
      #include <sys/io.h>
    #endif
#if ASM_IOCTL_H
    #include <asm/ioctl.h>
#endif
#if ASM_IO_H
    #include <asm/io.h>
#endif
    #define IO_FREE  0   /* To free I/O access proc. #*/
    #define IO_KEEP  1   /* To keep I/O access proc. #*/
  #endif
  #ifdef FreeBSD
    #include <fcntl.h>
    #include <machine/cpufunc.h>
  #endif
#endif

#ifdef PCAT
  #define COM1_BASE     (0x000003f8L)
  #define COM2_BASE     (0x000002f8L)
  #define COM3_BASE     (0x000003e8L)
  #define COM4_BASE     (0x000002e8L)
  #define SIO_MDM       4       /* rw modem control */
  #define SIO_MDM_RTS   0x02    /* RTS */
  #define SIO_MDM_DTR   0x01    /* DTR */
  #define SIO_MSR       6       /* rw modem status */
  #define SIO_MDMS_RLSD 0x80    /* RLSD */
  #define SIO_LCR       3       /* rw line contorlo */
  #define SIO_BRK       0x40    /*  break data send                      */
#endif /* PCAT */
/*------------------------*/
#ifdef  PC98
  #define SIO_RES 0x40          /* Reset Command */
  #define SIO_8N  0x4e          /* 8bit non parity 16clk */

  /* 8251 command */
  #define SIO_TX_EN 0x01    /*  0=TX-Disable 1=TX-Enable            */
  #define SIO_DTR   0x02    /*  ~DTR                                */
  #define SIO_RX_EN 0x04    /*  0=RX-Disable 1=RX-Enable            */
  #define SIO_BRK   0x08    /*  0=Normal 1=Break(TX-Status=0)       */
  #define SIO_ECL   0x10    /*  0=NOP 1=Clear Error Flag            */
  #define SIO_RTS   0x20    /*  ~RTS                                */
  #define SIO_SRES  0x40    /*  0=NOP 1=Reset                       */
  #define SIO_EH    0x80    /*  0=NOP 1=Enter Hunt Phase            */

  /* 8251 status  */
  #define SIO_TX_RDY   0x01    /*  0=TX-Buffer-Full 1=Empty         */
  #define SIO_RX_RDY   0x02    /*  0=No RX Data 1=Exists RX Data    */
  #define SIO_TX_EMP   0x04    /*  0=Exists TX Data 1=No TX Data    */
  #define SIO_PE       0x08    /*  0=No Parity Error 1=Parity Error */
  #define SIO_OVE      0x10    /*  0=No Overrun Error 1=OV Error    */
  #define SIO_FE       0x20    /*  0=No Framing Error 1=FR Error    */
  #define SIO_SYNC_BRK 0x40    /*  0/1=Break Detect                 */
  #define SIO_DSR      0x80    /*  ~DSR Status                      */

  /*; 8251a 2-12-1*/
  #define SIOD  0x30
  #define SIOC  0x32
  #define PPI1B 0x33        /*
  ;     pb0     calender read line
  ;     pb1     external ram parity error detecter
  ;     pb2     none
  ;     pb3     crt type 1=high, 0=normal
  ;     pb4     int3(HD)
  ;     pb5     CD carrier detect(negative)
  ;     pb6     CTS (negative)
  ;     pb7     CI (rs232c ?) */
  #define DSR_MASK 0x10  /* Put here 8251 MSB */
  #define CD_MASK  0x20
  #define CTS_MASK 0x40
  #define CI_MASK  0x80
#endif  /* for PC98 */

/*==================================================*/
/* I/O Port Access Functions' Aliases.             #*/
/* Caution : 'outb' in Linux :Parameters specified #*/
/*            in opposite order against others.    #*/
/*==================================================*/
#ifdef GCC
  #ifdef LINUX
    #define outportb(addr,val) outb(val,addr)
  #endif
  #ifdef FreeBSD
    #define outportb outb
  #endif
  #define inportb inb
  /* Define Number of Ports which is used per 1 COM Port. #*/
  #define NUM_PORTS 7L  
#endif

#ifdef MSC
  #define outportb  outp
  #define inportb inp
#endif

/*===============================================*/
int  ComPort ; /* COM Port No. ( 1 to 4 ) #*/
/*===============================================*/
/*  #include <dos.h>  #*/
int GbDOut(int Command);
void  GbSetComport(int  c);

/* Free I/O Permission for PC-UNIX. #*/
void   GbFree( void ) ;

/* Xfer PortNo(1to4) to PortAddress(0x3f8/2f8/3e8/2e8) #*/ 
unsigned long PtoA( int pno ) ;
/*===============================================*/

/* Define Static Valuables */
#ifdef  PCAT
  static  int GbComport ;  /* Port Address 0x3f8/2f8/3e8/2e8 #*/
#endif

#ifdef  PC98
  static  unsigned char ComMem;
#endif

#ifdef FreeBSD
  static int Fd ; /* File Descriptor */
#endif

/* This array is moved to Global Area. #*/
static  int chn[8] = { 0x00, 0x40, 0x10, 0x50, 0x20, 0x60, 0x30, 0x70 };

#define UNIPOLA 0x08
#define BIPOLA  0
static  int GbPola ;

static  int GbCh ;

#define Diff  0
#define Singl 0x04
static  int GbSglDif ;
static  int GbTrigaPola ;   /* 1=Minus to Plus / -1=Plus to Minus */
static  int GbTrigaAuto ;   /* 1=Auto / else Normal         */
static  long  GbTW;         /* Time Wait            */
static  int GbTxStat;       /* TX Break Status */

#define MAX_COMPORT  4

/*===============================================*/
/* Please set 'ComPort'(1 to 4) before calling this procedure. #*/
void  GbInit( void )
{
    int ret ;

/* Check COM Port No. Extent */
    if (( ComPort <= 0 )  ||  ( ComPort > MAX_COMPORT ))
       {
       fprintf( stderr, "\nIllegal value 'ComPort'(0x%04x) specified in 'GbInit'.\n", ComPort ) ; 
       exit( 1 ) ;
       }

/* Get Permission of All addresses from each COM Port. #*/
#ifdef  GCC
  #ifdef  LINUX
    if ( ComPort == 1 )
       {
       ret = ioperm( COM1_BASE, NUM_PORTS, IO_KEEP ) ;
       if ( ret != 0 )
          {
          fprintf( stderr, "\nCannot Access %04lx-%04lx in 'GbInit'.\n", COM1_BASE, COM1_BASE + NUM_PORTS ) ; 
          exit( 1 ) ;
          }
       }
    if ( ComPort == 2 )
       {
       ret = ioperm( COM2_BASE, NUM_PORTS, IO_KEEP ) ;
       if ( ret != 0 )
          {
          fprintf( stderr, "\nCannot Access %04lx-%04lx in 'GbInit'.\n", COM2_BASE, COM2_BASE + NUM_PORTS ) ; 
          exit( 1 ) ;
          }
       }
    if ( ComPort == 3 )
       {
       ret = ioperm( COM3_BASE, NUM_PORTS, IO_KEEP ) ;
       if ( ret != 0 )
          {
          fprintf( stderr, "\nCannot Access %04lx-%04lx in 'GbInit'.\n", COM3_BASE, COM3_BASE + NUM_PORTS ) ; 
          exit( 1 ) ;
          }
       }
    if ( ComPort == 4 )
       {
       ret = ioperm( COM4_BASE, NUM_PORTS, IO_KEEP ) ;
       if ( ret != 0 )
          {
          fprintf( stderr, "\nCannot Access %04lx-%04lx in 'GbInit'.\n", COM4_BASE, COM4_BASE + NUM_PORTS ) ; 
          exit( 1 ) ;
          }
       }
  #endif
  #ifdef  FreeBSD
     if (( Fd = open("/dev/io", O_RDWR)) == -1) /* Read/Write Mode */
        {
        fprintf( stderr, "\nCannot Access I/O port in 'GbInit'.\n") ; 
        exit( 1 ) ;
        }
  #endif
#endif

#ifdef  PCAT
  GbComport = PtoA(ComPort) ;
#endif  /* PCAT */
#ifdef  PC98
  ComPort = 1
#endif

  /* GID-ADC Default Value */
  GbPola = BIPOLA;  GbCh = 0;  GbSglDif = Singl;
  GbTrigaPola = 1;  GbTrigaAuto = 1;  GbTW = 1;
}
/*--------------------------*/
/* COM Port No (1 to 4) */
/* #pragma argsused */
void  GbInitPower()
{
  long  t ;

  if (( ComPort <= 0 ) || ( ComPort > MAX_COMPORT ))
     {
     fprintf( stderr, "\nIllegal arg 'ComPort'(0x%04x) specified in 'GbInitPower'.\n", ComPort ) ;
     GbFree() ;
     exit( 1 ) ;
     }

#ifdef  PC98
  unsigned int tn;
  outportb(SIOC, 0);
  for(tn=0xffff; tn>0; tn--);
  outportb(SIOC, 0);
  for(tn=0xffff; tn>0; tn--);
  outportb(SIOC, SIO_RES);
  for(tn=0xffff; tn>0; tn--);
  outportb(SIOC, SIO_8N);
  for(tn=0xffff; tn>0; tn--);
  outportb(SIOC, ComMem = SIO_RTS | SIO_DTR);
#endif  /* PC98 */

#ifdef  PCAT
  GbSetComport( ComPort ) ;
  outportb(GbComport + SIO_MDM, 0);
  outportb(GbComport + SIO_MDM, SIO_MDM_RTS | SIO_MDM_DTR);
#endif  /* PCAT */

  GbTxStat=0 ;
  for ( t = INIT_DMY_LOOP ;  t > 0 ;  t-- )
      ;   /* Wait until Power Stable. */
  GbDOut( 0x86 ) ;  /* Specify Internal Clock Mode. */
}
/*--------------------------*/
/*COM Port No (1 to 4) */
/* #pragma argsused #*/
void  GbSetComport( int  c )
{

  if (( c <= 0 ) || ( c > MAX_COMPORT ))
     {
     fprintf( stderr, "\nIllegal arg 'c'(0x%04x) specified in 'GbSetComPort'.\n", c ) ;
     GbFree() ;
     exit( 1 ) ;
     }

#ifdef  PCAT
  /* Changed to call function 'PtoA' to get addr. #*/
  GbComport = PtoA( c ) ;
#endif  /* PCAT */
}

/*--------------------------*/
void  GbSetCh(int  ch)
{
  GbCh = chn[ch & 0x07];
}

/*--------------------------*/
void  GbSetUnipola(void)  { GbPola = UNIPOLA; }
void  GbSetBipola(void){ GbPola = BIPOLA;  }
void  GbSetSingle(void) { GbSglDif = Singl; }
void  GbSetDiff(void)   { GbSglDif = Diff; }
void  GbSetTrigaNone(void)  { GbTrigaPola = 0;  }
void  GbSetTrigaPlus(void)  { GbTrigaPola = 1;  }
void  GbSetTrigaMinus(void)  { GbTrigaPola = -1;  }
void  GbSetTWait(long t)  { GbTW = t;   if(GbTW <=0) GbTW=1;}
long  GbGetTWait(void)  { return  GbTW; }

/* pol = BIPOLA/UNIPOLA */
void  GbSetPola(int pol)
{
  switch( pol )
    {
    case UNIPOLA:
        GbSetUnipola() ;
        break ;
    case BIPOLA:
        GbSetBipola() ;
        break ;
    default :
        fprintf( stderr, "\nInvalid value 'pol'(0x%04x) specified in 'GbSetPola'.\n", pol ) ;
        GbFree() ;
        exit( 1 ) ;
        break;
    }
}

/*-----------------------------------*/
/*long  GetTime55ms(void) { return  biostime(0, 0L);  }*/
/*-----------------------------------*/
/* Get 1 Sample Data */
int GbCnv(void)
{
  int m,  rd;
  m = 0x80 | GbCh | GbPola | GbSglDif | 0x01;

  rd = GbDOut(m);
  if(GbPola == BIPOLA)  { if(rd > 2047) rd = rd - 4096; }
  return  rd;
}
/*-----------------------------------*/
/* Get Data for Specified Times, Trigger On / Error Ret when retry failed. */
/* #pragma argsused */
int GbNTCnv(int v,  int  n,  int wn, int *buf, long *tbuf)
{
  int w;
  if(GbTrigaAuto && wn) w = n*2;  else  w = 10000;
  if(GbTrigaPola==1) {
    for(; GbCnv() > v && w > 0;  w--);    
    for(; GbCnv() < v && w > 0;  w--);
  } else  if(GbTrigaPola<1){
    for(; GbCnv() < v && w > 0;  w--);    
    for(; GbCnv() > v && w > 0;  w--);
  }
  if(w<=0 && !GbTrigaAuto)  return  1;

/*asm { pushf;   cli; }*/
  for(; n>0;  n--)  {
    *buf++ = GbCnv();   /* if(tbuf) *tbuf++ = GetTime55ms(); */
  }
/*asm { popf; } */

  return  0;
}
/*-----------------------------------*/
void  GbTWait(void)
{
  long n;
  for(n = GbTW; n>0;  n--);
}
/*-----------------------------------*/
int GbSCLK(int  Dout)
{
  int rd;

#ifdef  PCAT
  /* data out */
  outportb(GbComport + SIO_MDM, ((Dout)? 0 : SIO_MDM_RTS) | SIO_MDM_DTR);
  GbTWait();
  /* data in */
  rd = inportb(GbComport + SIO_MSR) & SIO_MDMS_RLSD;
  /* sclk 1 */
  outportb(GbComport + SIO_MDM, ((Dout)? 0 : SIO_MDM_RTS));
  GbTWait();
  /* sclk 0 */
  outportb(GbComport + SIO_MDM, SIO_MDM_RTS | SIO_MDM_DTR);
  GbTWait();
#endif

#ifdef  PC98
  /* data out */
  outportb(SIOC,  ((Dout)? 0 : SIO_RTS) | SIO_DTR);
  GbTWait();
  /* data in */
  rd = ~(inportb(PPI1B)) & CD_MASK;
  /* sclk 1 */
  outportb(SIOC, ((Dout)? 0 : SIO_RTS));
  GbTWait();
  /* sclk 0 */
  outportb(SIOC, SIO_RTS | SIO_DTR);
  GbTWait();
#endif
  return  rd;
}
/*-----------------------------------*/
/* Set Final Status after Command Output to ADC */
void  GbSetTxE(void)
{
#ifdef  PCAT
  if ( GbTxStat )
     {
     outportb(GbComport + SIO_MDM, SIO_MDM_DTR);   /* RTS=0 */ 
     }
  else
     {
     outportb(GbComport+SIO_MDM, SIO_MDM_RTS | SIO_MDM_DTR); /* RTS=1 */ 
     }
#endif  /* PCAT */

#ifdef  PC98
  outportb(SIOC, ComMem);
#endif
}
/*-----------------------------------*/
/* It cannot be used from BIOS when xfer, because set Minus to DIN.
  i=0 Normal  i!=0 Tx=Hi-LEVEL
*/
void  GbSetTx(int i)
{
#ifdef  PCAT
  if ( i )
     {
     outportb(GbComport + SIO_LCR, SIO_BRK);    /* Set Break Signal */
     outportb(GbComport + SIO_MDM, SIO_MDM_DTR);   /* RTS=0 */
     }
  else
     {
     outportb(GbComport + SIO_LCR, 0);
     outportb(GbComport + SIO_MDM, SIO_MDM_RTS | SIO_MDM_DTR);   /* RTS=1 */
     }
#endif  /* PCAT */

#ifdef  PC98
  if(i) {
    ComMem = SIO_BRK | SIO_DTR;  /* Set Break Signal */
  } else  {
    ComMem = SIO_DTR | SIO_RTS;  /* Reset Break Signal */
  }
  outportb(SIOC, ComMem);
#endif  /* PC98 */

  GbTxStat = i;
}
/*-----------------------------------*/
/* Set TX Signal to 1 while t-Loop period. */
void  GbSetTxPulse(long t)
{
  GbSetTx(1);
  for(; t>0;  t--);
  GbSetTx(0);
}
/*-----------------------------------*/
/* Set TX to 1, reading Analog ch until data expired. Returns 0 exceeds extent. */
/* Pretest Execution sometimes disturb pulse. */
void  GbSetTxPulseWait(int d)
{
  if(d >= GbCnv())  {
    GbSetTx(1);
    while(d >= GbCnv());
    GbSetTx(0);
  }
}
/*-----------------------------------*/
int GbDOut(int Command)
{
  int n,  rd;

  /* Command Output */
  for(n=8;  n>0;  n--)  { rd = GbSCLK(Command & 0x80);  Command <<= 1;  }
  /* Get DATA */
  for(rd=0,n=13;  n>0;  n--)
  { rd = (rd << 1) | (GbSCLK(0)? 1 : 0);  }

  /* Set Final Status depend on TX Status for the sake of maintain Minus Power Supply. */
  GbSetTxE();
  return  rd & 0xfff;
}
/*-----------------------------------*/

/*====================================================*/
/* Following Functions are only for LINUX Service.   #*/          
/*====================================================*/
/* Free Permission of I/O access. (submit to kernel) #*/
/* Args : None #*/
/* Ret  : None #*/
void   GbFree( void )
{
#ifdef PCAT
  #ifdef LINUX
    if ( ComPort == 1 )
       ioperm( COM1_BASE, NUM_PORTS, IO_FREE ) ;
    if ( ComPort == 2 )
       ioperm( COM2_BASE, NUM_PORTS, IO_FREE ) ;
    if ( ComPort == 3 )
       ioperm( COM3_BASE, NUM_PORTS, IO_FREE ) ;
    if ( ComPort == 4 )
       ioperm( COM4_BASE, NUM_PORTS, IO_FREE ) ;
  #endif
  #ifdef FreeBSD
     close(Fd);
  #endif
#endif
}


/* Exchange COM Port No. to Port Address. #*/
/* Args : int pno : COM PortNo (1 to 4) #*/
/* Ret  : unsigned long : 0x3f8/2f8/3e8/2e8 #*/
/*        If not PCAT, error returns 0L #*/ 
unsigned long PtoA( int pno )
{
#ifdef PCAT
    switch ( pno )
       {
       case  1 :
               return( COM1_BASE ) ;
               break ;
       case  2 :
               return( COM2_BASE ) ;
               break ;
       case  3 :
               return( COM3_BASE ) ;
               break ;
       case  4 :
               return( COM4_BASE ) ;
               break ;
       default :
               fprintf( stderr, "\nIllegal arg pno(0x%04x) specified in 'PtoA'.\n", pno ) ;
               GbFree() ;
               exit(1) ;
               break ;
       }
#else
    return( 0L ) ;
#endif
}

/*==========================*/
/*  End  of  gidadcctl.c    */
/*==========================*/
